@@ -1,12 +1,12 @@
 import totem
-import gobject, gtk
+import gobject, gtk, gconf
 import gdata.service
 import urllib
 import httplib
 import atom
 import threading
 import re
-from os import unlink
+import os
 
 class DownloadThread (threading.Thread):
 	def __init__ (self, youtube, url, treeview_name):
@@ -18,7 +18,7 @@ class DownloadThread (threading.Thread):
 		self.youtube.entry_lock.acquire (True)
 		try:
 			self.youtube.entry[self.treeview_name] = self.youtube.service.Get (self.url).entry
-		except RequestError:
+		except gdata.service.RequestError:
 			"""Probably a 503 service unavailable. Unfortunately we can't give an error message, as we're not in the GUI thread"""
 			"""Just let the lock go and return"""
 		self.youtube.entry_lock.release ()
@@ -45,10 +45,15 @@ class YouTube (totem.Plugin):
 
 		self.vadjust = {}
 		self.liststore = {}
+		self.treeview = {}
 	def activate (self, totem_object):
 		"""Check for the availability of the flvdemux and souphttpsrc GStreamer plugins"""
 		bvw_name = totem_object.get_video_widget_backend_name ()
 
+		"""If the user's selected 1.5Mbps or greater as their connection speed, grab higher-quality videos
+		   and drop the requirement for the flvdemux plugin."""
+		self.gconf_client = gconf.client_get_default ()
+
 		if bvw_name.find ("GStreamer") != -1:
 			try:
 				import pygst
@@ -56,13 +61,13 @@ class YouTube (totem.Plugin):
 				import gst
 
 				registry = gst.registry_get_default ()
-				if registry.find_plugin ("flvdemux") == None or registry.find_plugin ("souphttpsrc") == None:
+				if (self.get_fmt_string () == "" and registry.find_plugin ("flvdemux") == None) or registry.find_plugin ("soup") == None:
 					"""This means an error will be displayed when they try to play anything"""
 					self.gstreamer_plugins_present = False
 			except ImportError:
 				"""Do nothing; either it's using xine or python-gstreamer isn't installed"""
 
-		"""Continue loading the plugin as before"""		
+		"""Continue loading the plugin as before"""
 		self.builder = self.load_interface ("youtube.ui", True, totem_object.get_main_window (), self)
 		self.totem = totem_object
 
@@ -70,6 +75,7 @@ class YouTube (totem.Plugin):
 		self.search_entry.connect ("activate", self.on_search_entry_activated)
 		self.search_button = self.builder.get_object ("yt_search_button")
 		self.search_button.connect ("clicked", self.on_search_button_clicked)
+		self.progress_bar = self.builder.get_object ("yt_progress_bar")
 
 		self.notebook = self.builder.get_object ("yt_notebook")
 		self.notebook.connect ("switch-page", self.on_notebook_page_changed)
@@ -100,6 +106,23 @@ class YouTube (totem.Plugin):
 		treeview.connect_after ("starting-video", self.on_starting_video)
 		treeview.insert_column_with_attributes (0, _("Videos"), renderer, thumbnail=0, title=1)
 
+		"""Add the extra popup menu options. This is done here rather than in the UI file, because it's done for multiple treeviews;
+		if it were done in the UI file, the same action group would be used multiple times, which GTK+ doesn't like."""
+		ui_manager = treeview.get_ui_manager ()
+		action_group = gtk.ActionGroup ("youtube-action-group")
+		action = gtk.Action ("open-in-web-browser", _("_Open in Web Browser"), _("Open the video in your web browser"), "gtk-jump-to")
+		action_group.add_action_with_accel (action, None)
+
+		ui_manager.insert_action_group (action_group, 1)
+		ui_manager.add_ui (ui_manager.new_merge_id (),
+				   "/ui/totem-video-list-popup/",
+				   "open-in-web-browser",
+				   "open-in-web-browser",
+				   gtk.UI_MANAGER_MENUITEM,
+				   False)
+		menu_item = ui_manager.get_action ("/ui/totem-video-list-popup/open-in-web-browser")
+		menu_item.connect ("activate", self.on_open_in_web_browser_activated)
+
 		self.vadjust[treeview_name] = treeview.get_vadjustment ()
 		self.vadjust[treeview_name].connect ("value-changed", self.on_value_changed)
 		vscroll = self.builder.get_object ("yt_scrolled_window_" + treeview_name).get_vscrollbar ()
@@ -107,6 +130,7 @@ class YouTube (totem.Plugin):
 		vscroll.connect ("button-release-event", self.on_button_release_event)
 
 		self.liststore[treeview_name] = self.builder.get_object ("yt_liststore_" + treeview_name)
+		self.treeview[treeview_name] = treeview
 		treeview.set_model (self.liststore[treeview_name])
 	def on_notebook_page_changed (self, notebook, notebook_page, page_num):
 		self.current_treeview_name = self.notebook_pages[page_num]
@@ -119,7 +143,13 @@ class YouTube (totem.Plugin):
 		self.youtube_id = youtube_id
 		self.start_index["related"] = 1
 		self.results["related"] = 0
+		self.progress_bar.set_text (_("Fetching related videos..."))
 		self.get_results ("/feeds/api/videos/" + urllib.quote (youtube_id) + "/related?max-results=" + str (self.max_results), "related")
+	def get_fmt_string (self):
+		if self.gconf_client.get_int ("/apps/totem/connection_speed") >= 10:
+			return "&fmt=18"
+		else:
+			return ""
 	def check_url_for_redirects (self, url_path):
 		try:
 			conn = httplib.HTTPConnection ("www.youtube.com")
@@ -152,7 +182,8 @@ class YouTube (totem.Plugin):
 		location = self.check_url_for_redirects ("/v/" + urllib.quote (youtube_id))
 
 		if location != False:
-			mrl = "http://www.youtube.com/get_video?video_id=" + urllib.quote (youtube_id) + "&t=" + urllib.quote (re.match (".*[?&]t=([^&]+)", location).groups ()[0])
+			mrl = "http://www.youtube.com/get_video?video_id=" + urllib.quote (youtube_id) + "&t=" + urllib.quote (re.match (".*[?&]t=([^&]+)", location).groups ()[0]) + self.get_fmt_string ()
+
 			"""location_check = self.check_url_for_redirects (mrl)
 			if location_check != False:
 				mrl = location_check
@@ -166,6 +197,13 @@ class YouTube (totem.Plugin):
 			model.set_value (iter, 2, mrl)
 
 		return True
+	def on_open_in_web_browser_activated (self, action):
+		model, rows = self.treeview[self.current_treeview_name].get_selection ().get_selected_rows ()
+		iter = model.get_iter (rows[0])
+		youtube_id = model.get_value (iter, 3)
+
+		"""Open the video in the browser"""
+		os.spawnlp (os.P_NOWAIT, "xdg-open", "xdg-open", "http://www.youtube.com/watch?v=" + urllib.quote (youtube_id) + self.get_fmt_string ())
 	def on_button_press_event (self, widget, event):
 		self.button_down = True
 	def on_button_release_event (self, widget, event):
@@ -175,6 +213,7 @@ class YouTube (totem.Plugin):
 		"""Load more results when we get near the bottom of the treeview"""
 		if not self.button_down and (adjustment.get_value () + adjustment.page_size) / adjustment.upper > 0.8 and self.results[self.current_treeview_name] >= self.max_results:
 			self.results[self.current_treeview_name] = 0
+			self.progress_bar.set_text (_("Fetching more videos..."))
 			if self.current_treeview_name == "search":
 				self.get_results ("/feeds/api/videos?vq=" + urllib.quote_plus (self.search_terms) + "&max-results=" + str (self.max_results) + "&orderby=relevance&start-index=" + str (self.start_index["search"]), "search", False)
 				if self.debug:
@@ -189,6 +228,7 @@ class YouTube (totem.Plugin):
 	def populate_list_from_results (self, treeview_name):
 		"""Check and acquire the lock"""
 		if self.entry_lock.acquire (False) == False:
+			self.progress_bar.pulse ()
 			return True
 
 		"""Return if there are no results (or we've finished)"""
@@ -196,6 +236,8 @@ class YouTube (totem.Plugin):
 			"""Revert the cursor"""
 			window = self.vbox.window
 			window.set_cursor (None)
+			self.progress_bar.set_fraction (0.0)
+			self.progress_bar.set_text ("")
 
 			self.entry[treeview_name] = None
 			self.entry_lock.release ()
@@ -208,6 +250,9 @@ class YouTube (totem.Plugin):
 		self.start_index[treeview_name] += 1
 		youtube_id = self.convert_url_to_id (entry.id.text)
 
+		"""Update the progress bar"""
+		self.progress_bar.set_fraction (float (self.results[treeview_name]) / float (self.max_results))
+
 		self.entry_lock.release ()
 
 		"""Find the content tag"""
@@ -225,17 +270,17 @@ class YouTube (totem.Plugin):
 		try:
 			filename, headers = urllib.urlretrieve (thumbnail_url)
 		except IOError:
-			print "Could not load thumbnail " + thumbnail_url + " for video."
+			print "Could not retrieve thumbnail " + thumbnail_url + " for video."
 			return True
 
 		try:
 			pixbuf = gtk.gdk.pixbuf_new_from_file (filename)
 		except gobject.GError:
-			print "Could not load thumbnail " + filename + " for video. It has been left in place for investigation."
+			print "Could not open thumbnail " + filename + " for video. It has been left in place for investigation."
 			return True
 
 		"""Don't leak the temporary file"""
-		unlink (filename)
+		os.unlink (filename)
 
 		self.liststore[treeview_name].append ([pixbuf, entry.title.text, mrl, youtube_id])
 
@@ -252,6 +297,7 @@ class YouTube (totem.Plugin):
 		self.search_terms = search_terms
 		self.start_index["search"] = 1
 		self.results["search"] = 0
+		self.progress_bar.set_text (_("Fetching search results..."))
 		self.get_results ("/feeds/api/videos?vq=" + urllib.quote_plus (search_terms) + "&orderby=relevance&max-results=" + str (self.max_results), "search")
 	def on_search_entry_activated (self, entry):
 		self.search_button.clicked ()
@@ -265,6 +311,7 @@ class YouTube (totem.Plugin):
 		"""Give us a nice waiting cursor"""
 		window = self.vbox.window
 		window.set_cursor (gtk.gdk.Cursor (gtk.gdk.WATCH))
+		self.progress_bar.pulse ()
 
 		self.results_downloaded = False
 		DownloadThread (self, url, treeview_name).start ()
