@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301  USA.
 
    Author: Bastien Nocera <hadess@hadess.net>
  */
@@ -24,9 +24,8 @@
 #define TOTEM_PLAYLIST_H
 
 #include <gtk/gtkvbox.h>
-#include <libgnomevfs/gnome-vfs-volume.h>
-
-#include "totem-pl-parser.h"
+#include <totem-pl-parser.h>
+#include <gio/gio.h>
 
 G_BEGIN_DECLS
 
@@ -51,9 +50,14 @@ typedef struct TotemPlaylist	       TotemPlaylist;
 typedef struct TotemPlaylistClass      TotemPlaylistClass;
 typedef struct TotemPlaylistPrivate    TotemPlaylistPrivate;
 
+typedef void (*TotemPlaylistForeachFunc) (TotemPlaylist *playlist,
+					  const gchar   *filename,
+					  const gchar   *uri,
+					  gpointer       user_data);
+
 struct TotemPlaylist {
 	GtkVBox parent;
-	TotemPlaylistPrivate *_priv;
+	TotemPlaylistPrivate *priv;
 };
 
 struct TotemPlaylistClass {
@@ -65,13 +69,15 @@ struct TotemPlaylistClass {
 	void (*current_removed) (TotemPlaylist *playlist);
 	void (*repeat_toggled) (TotemPlaylist *playlist, gboolean repeat);
 	void (*shuffle_toggled) (TotemPlaylist *playlist, gboolean toggled);
+	void (*subtitle_changed) (TotemPlaylist *playlist);
+	void (*item_added) (TotemPlaylist *playlist, const gchar *filename, const gchar *uri);
+	void (*item_removed) (TotemPlaylist *playlist, const gchar *filename, const gchar *uri);
 };
 
 GtkType    totem_playlist_get_type (void);
 GtkWidget *totem_playlist_new      (void);
 
 /* The application is responsible for checking that the mrl is correct
- * Handles directories, m3u playlists, and shoutcast playlists
  * @display_name is if you have a preferred display string for the mrl,
  * NULL otherwise
  */
@@ -90,20 +96,15 @@ void totem_playlist_save_current_playlist_ext (TotemPlaylist *playlist,
 /* totem_playlist_clear doesn't emit the current_removed signal, even if it does
  * because the caller should know what to do after it's done with clearing */
 gboolean   totem_playlist_clear (TotemPlaylist *playlist);
-void       totem_playlist_clear_with_prefix (TotemPlaylist *playlist,
-					     const char *prefix);
-void       totem_playlist_clear_with_gnome_vfs_volume (TotemPlaylist *playlist,
-						       GnomeVFSVolume *volume);
-char      *totem_playlist_get_current_mrl (TotemPlaylist *playlist);
+void       totem_playlist_clear_with_g_mount (TotemPlaylist *playlist,
+					      GMount *mount);
+char      *totem_playlist_get_current_mrl (TotemPlaylist *playlist,
+					   char **subtitle);
 char      *totem_playlist_get_current_title (TotemPlaylist *playlist,
 					     gboolean *custom);
 char      *totem_playlist_get_title (TotemPlaylist *playlist,
 				     guint index);
 
-gboolean   totem_playlist_get_current_metadata (TotemPlaylist *playlist,
-						char         **artist,
-						char         **title,
-						char         **album);
 gboolean   totem_playlist_set_title (TotemPlaylist *playlist,
 				     const char *title,
 				     gboolean force);
@@ -124,6 +125,7 @@ void       totem_playlist_set_shuffle (TotemPlaylist *playlist,
 				       gboolean shuffle);
 
 gboolean   totem_playlist_set_playing (TotemPlaylist *playlist, TotemPlaylistStatus state);
+TotemPlaylistStatus totem_playlist_get_playing (TotemPlaylist *playlist);
 
 void       totem_playlist_set_at_start (TotemPlaylist *playlist);
 void       totem_playlist_set_at_end (TotemPlaylist *playlist);
@@ -132,6 +134,10 @@ guint      totem_playlist_get_current (TotemPlaylist *playlist);
 guint      totem_playlist_get_last (TotemPlaylist *playlist);
 void       totem_playlist_set_current (TotemPlaylist *playlist, guint index);
 
+void       totem_playlist_foreach (TotemPlaylist *playlist,
+				   TotemPlaylistForeachFunc callback,
+				   gpointer user_data);
+
 G_END_DECLS
 
 #endif /* TOTEM_PLAYLIST_H */
