@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301  USA.
 
    Author: Bastien Nocera <hadess@hadess.net>
  */
@@ -41,8 +41,8 @@ cb_resize (Totem * totem)
 	h = totem->win->allocation.height;
 
 	g_value_init (&gvalue_size, G_TYPE_INT);
-	pane = glade_xml_get_widget (totem->xml, "tmw_main_pane");
-	gtk_widget_style_get_property(pane, "handle-size", &gvalue_size);
+	pane = GTK_WIDGET (gtk_builder_get_object (totem->xml, "tmw_main_pane"));
+	gtk_widget_style_get_property (pane, "handle-size", &gvalue_size);
 	handle_size = g_value_get_int (&gvalue_size);
 	
 	if (totem->sidebar_shown) {
@@ -73,10 +73,6 @@ totem_sidebar_toggle (Totem *totem, gboolean state)
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), state);
 	totem_signal_unblock_by_data (G_OBJECT (action), totem);
 
-	gconf_client_set_bool (totem->gc,
-				GCONF_PREFIX"/sidebar_shown",
-				state,
-				NULL);
 	totem->sidebar_shown = state;
 	cb_resize(totem);
 }
@@ -96,10 +92,10 @@ totem_sidebar_is_visible (Totem *totem)
 void
 totem_sidebar_setup (Totem *totem, gboolean visible, const char *page_id)
 {
-	GtkWidget *item;
+	GtkPaned *item;
 	GtkAction *action;
 
-	item = glade_xml_get_widget (totem->xml, "tmw_main_pane");
+	item = GTK_PANED (gtk_builder_get_object (totem->xml, "tmw_main_pane"));
 	totem->sidebar = ev_sidebar_new ();
 	ev_sidebar_add_page (EV_SIDEBAR (totem->sidebar),
 			"playlist", _("Playlist"),
@@ -111,7 +107,7 @@ totem_sidebar_setup (Totem *totem, gboolean visible, const char *page_id)
 		ev_sidebar_set_current_page (EV_SIDEBAR (totem->sidebar),
 				"playlist");
 	}
-	gtk_paned_pack2 (GTK_PANED (item), totem->sidebar, FALSE, FALSE);
+	gtk_paned_pack2 (item, totem->sidebar, FALSE, FALSE);
 
 	totem->sidebar_shown = visible;
 
@@ -131,16 +127,24 @@ totem_sidebar_setup (Totem *totem, gboolean visible, const char *page_id)
 		gtk_widget_hide (totem->sidebar);
 }
 
-const char *
+char *
 totem_sidebar_get_current_page (Totem *totem)
 {
+	if (totem->sidebar == NULL)
+		return NULL;
 	return ev_sidebar_get_current_page (EV_SIDEBAR (totem->sidebar));
 }
 
 void
-totem_sidebar_set_current_page (Totem *totem, const char *name)
+totem_sidebar_set_current_page (Totem *totem,
+				const char *name,
+				gboolean force_visible)
 {
+	if (name == NULL)
+		return;
+
 	ev_sidebar_set_current_page (EV_SIDEBAR (totem->sidebar), name);
-	totem_sidebar_toggle (totem, TRUE);
+	if (force_visible != FALSE)
+		totem_sidebar_toggle (totem, TRUE);
 }
 
