@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301  USA.
 
    Author: Bastien Nocera <hadess@hadess.net>
  */
@@ -50,7 +50,7 @@ totem_playing_dvd (const char *uri)
 }
 
 static void
-totem_ensure_dot_dir (const char *path)
+totem_ensure_dir (const char *path)
 {
 	if (g_file_test (path, G_FILE_TEST_IS_DIR) != FALSE)
 		return;
@@ -64,7 +64,7 @@ totem_dot_dir (void)
 	static char *totem_dir = NULL;
 
 	if (totem_dir != NULL) {
-		totem_ensure_dot_dir (totem_dir);
+		totem_ensure_dir (totem_dir);
 		return totem_dir;
 	}
 
@@ -72,7 +72,26 @@ totem_dot_dir (void)
 				      "totem",
 				      NULL);
 
-	totem_ensure_dot_dir (totem_dir);
+	totem_ensure_dir (totem_dir);
+
+	return (const char *)totem_dir;
+}
+
+const char *
+totem_data_dot_dir (void)
+{
+	static char *totem_dir = NULL;
+
+	if (totem_dir != NULL) {
+		totem_ensure_dir (totem_dir);
+		return totem_dir;
+	}
+
+	totem_dir = g_build_filename (g_get_user_data_dir (),
+				      "totem",
+				      NULL);
+
+	totem_ensure_dir (totem_dir);
 
 	return (const char *)totem_dir;
 }
